#!/usr/bin/env python2
# -*- coding: utf-8 -*-

from psychopy import visual, core, gui, data, event, parallel
import time, datetime
import numpy as np
import sounddevice as sd
import soundfile as sf
from os.path import join, expanduser, isfile, isdir
from os import mkdir
from mn import create_click as click
import matplotlib.pyplot as plt
from StairAnalysis import load_data, plot_stair_prf, plot_val_prf
from plot_awareness_per_block import plot_awareness_per_block
import instr

# EDITS MADE FOR DEBUG

# change between computers
HZ = 144
nr_channels = 1 # 0 = 2, 1 = 4
debug = 0

experiment = 'AAN_LRclick'
#wd = join(expanduser('~'), 'Box Sync', experiment, 'Python')
wd = join(expanduser('~'), 'desktop', experiment, 'python')

# Parallel port
# p_port = parallel.ParallelPort(address = "0x4020") # room 225
p_port = parallel.ParallelPort(address='0x3028') # room 223

# Global variables
logdir = join(wd, 'logs')
ch = [[1,2],[3,4,1,2]]
#ch = [[3,4],[3,4,1,2]]
if nr_channels == 1:
    sd.default.device = "ASIO Fireface USB"
print('Sound device ------------------------------')
print(sd.default.device)
FS = 48000

def is_file(file):
    if isfile(file):
        print('File exists!\n%s' % file)
    assert(isfile(file) == False)

def change_type(x, types):
    '''Format list of strings as list of given types.'''
    return [eval("%s('%s')" % (t,v)) for t, v in zip(types,x)]

def subset(a,b,c):
    '''Subset values from one array on the basis of the values of another array.'''
    return(np.array([a])[np.array([b])==c])

def QUIT():
    print('Escape to quit.')
    core.quit()

def sigmoid(x, mid, slope):
    '''Generate random responses from a sigmoidal function.'''
    return(1 / (1 + np.exp( -slope * (x-mid) )))

def check4esc():
    '''Check for escape.'''
    keys = event.getKeys(keyList = ['escape'])
    if len(keys)>0 and 'escape' in keys: QUIT()

def add_log(f, start, end, FP, valnr, task):
    '''Add stuff at start of log file'''
    f = open(f,'r+')
    content = f.read()
    f.seek(0, 0)
    f.write(start.rstrip('\r\n') + '\n' + end.rstrip('\r\n') + '\nfp: ' +
        str(FP) + '\nValnr: ' + str(valnr) + '\nTask: ' + task +
        '\n\n' + content)
    f.close()

def send_triggers(in_trig):
    '''Send trigger for 50 ms then turn port off'''
    p_port.setData(int(in_trig))
    core.wait(0.05)
    p_port.setData(255)

def ready(text='Hit space to continue!', button='space'):
    visual.TextStim(win, color='gray', height = 0.02,text = text).draw()
    win.flip()
    event.waitKeys(keyList=['space'])
    win.flip()

# Make blocks and pauses
def block_pause(trials, blocks):
    assert trials % blocks == 0
    block = np.repeat(np.arange(blocks),int(trials/blocks))
    pause = np.linspace(0,trials,blocks+1, dtype=int)[1:-1]-1
    return block, pause

# Pause def -------------------------------------------------------------------
def paus(pauses,t,block):
    '''Makes pause at current trial'''
    if t in pauses:
        visual.TextStim(win, color='gray', height = 0.02,text = instr.pausInfo % (block[t]+1, max(block)+1)).draw()
        win.flip()
        event.waitKeys(keyList=['space'])
        win.flip()
        core.wait(1)

def trial(sound, trig):
    send_triggers(trig)
    #core.wait(0.01)
    for f in range(HZ): # 1000 ms fixation with click at 500 ms
        fixation.draw()
        # if f < int(HZ*0.05):
        #     diode.draw()
        if f == int(HZ*0.17):
            sd.play(sound, FS, mapping = ch[nr_channels])
        win.flip()

def make_trials(n, catch):
    np.random.seed()
    if catch:
        cond = [0] * 9 + [1] * 9 + [2] * 2 # 0 = left, 1 = right, 2 = catch
        corr = ['left'] * 9 + ['right'] * 9 + ['N'] * 2
        trig = [254] * 9 + [253] * 9 + [251] * 2
    else:
        cond = [0] * 5 + [1] * 5 # 0 = left, 1 = right
        corr = ['left'] * 5 + ['right'] * 5
        trig = [254] * 5 + [253] * 5

    x = [cond, corr, trig]
    a = [[] for i in range(len(x))]
    for mb in range(int(n/len(x[0]))):
        order = np.random.permutation(range(len(x[0])))
        for i in range(len(x)):
            a[i] += list(np.array(x[i])[np.array(order)])
    return a

def load_click():
    sgaindb = -10  # Level of signal re max
    sgain = 10**(sgaindb/20.0)  # Convert from dB
    s, fs = sf.read("E1.wav")
    s = s[0:4*48]  # only first 4 ms
    s = sgain * s
    return (s)

def obj_resp(cond, lvl, debug):
    # GET INPUT
    if debug == 1:
        if cond == 2:
            key = ['left', 'right'][np.random.randint(0,2)]
        else:
            correct = np.random.binomial(1, sigmoid(x = lvl, mid = -82, slope = 0.5))
            if correct == 1: key = ['left', 'right'][cond]
            elif correct == 0: key = ['left', 'right'][abs(cond-1)]
    elif debug == 0: 
        key = event.waitKeys(keyList=['left','right','escape'])[0]
    if key == 'escape': QUIT()
    if ['left', 'right', 'N'][cond] == key: correct = 1
    else: correct = 0
    return(correct, key)

def sub_resp(cond, lvl, debug):
    if debug == 1:
        aware = np.random.binomial(1, sigmoid(x = lvl, mid = -82, slope = 0.5))
        if aware == 1: key = 2
        elif aware == 0: key = 1
    elif debug == 0: 
        key = event.waitKeys(keyList=['1','2','3','escape'])[0]
        if key == 'escape': QUIT()
        if int(key) == 1: aware = 0
        else: aware = 1
    return(aware, key)

def write_log(columns, fileName, *log_data):
    data = ''.join(['{}\t' for i in range(len(columns.split('\t')))]).format(*log_data)
    with open(fileName, 'a') as f:
        f.write(data + '\n')

# Practice def ----------------------------------------------------------------
def procedure(n, b, log, proc, inst, lvll, lvlr, catch, valnr, FP, fp_logdir, task):
    if log: # store log file
        if proc=='Experiment':
            s = proc[:3]
            block, pauses = block_pause(n, b)
        elif proc=='Validation':
            s = '%s_%02d'% (proc[:3],valnr)
        fileName = join(fp_logdir, '%s_fp%02d_%s.txt' % (experiment, FP, s))
        is_file(fileName)
        start_time = '%s start: '%proc + datetime.datetime.now().strftime("%y-%m-%d %H:%M")
        columns = 'fp\ttrial\tcond\tlevel\tobj_key\tsub_key\tcorrect\taware\n'
        with open(fileName, 'w') as f:
            f.write(columns)
    correct_ = []
    aware_ = []
    cond, corr, trig = make_trials(n, catch) # create trials
    print('='* 5, proc, '='*(17-len(proc)))
    tmp = np.unique(cond, return_counts=True)
    for i in range(len(tmp[0])):
        print('Cond %i:\t%i trials' % (tmp[0][i], tmp[1][i]))
    print('='*24,'\n')

    # Generate auditory stimuli
    s = load_click()
    sounds = [click(s, -0.25, -8, lvll, nr_channels), # Left
              click(s, 0.25, 8, lvlr, nr_channels), # Right
              click(s, 0.25, 8, -400, nr_channels)] # catch
    # WARM UP SYSTEM WARM UP SYSTEM WARM UP SYSTEM WARM UP SYSTEM WARM UP SYSTEM
    if nr_channels == 0:
        sd.play(np.transpose((np.zeros(48*200),np.zeros(48*200))), FS, mapping=ch[nr_channels])
    elif nr_channels == 1:
        sd.play(np.transpose((np.zeros(48*200),np.zeros(48*200),np.zeros(48*200),np.zeros(48*200))), FS, mapping=ch[nr_channels])
    time.sleep(0.5)

    if inst==False:
        ready(button='space') # wait for space to start
    else:
        ready(inst, 'space')
    core.wait(1)
    # Trial loop --------------------------------------------------------------
    for t in range(n):
        if debug == 0:
            trial(sounds[cond[t]], trig[t])
            win.flip()
        # Get responses
            VT.text = instr.objInst
            VT.draw()
            win.flip()
        lvl = [lvll,lvlr,-400][cond[t]]
        correct, obj_key = obj_resp(cond[t], lvl, debug)
        correct_ += [correct]
        if debug == 0:
            VT.text = instr.subInst
            VT.draw()
            win.flip()
        aware, sub_key = sub_resp(cond[t], lvl, debug)
        aware_ += [aware]
        win.flip()
        # Write trial to logfile
        if log:
            write_log(columns, fileName, FP, t+1, int(cond[t]), lvl, obj_key, int(sub_key), correct, aware)
        if proc=='Experiment':
            paus(pauses,t,block)
        check4esc()
    if log:
        end_time = 'Experiment end: ' + datetime.datetime.now().strftime("%y-%m-%d %H:%M")
        add_log(fileName, start_time, end_time, FP, valnr, task)
    # give some output to user
    print('='*60)
    print('Aware L: %.1f\nAware R: %.1f' % (np.mean(subset(aware_,cond,0))*100, np.mean(subset(aware_,cond,1))*100))
    print('='*60)

# Staircase def ---------------------------------------------------------------
def staircase(n, FP, fp_logdir):
    fileName = join(fp_logdir, '%s_cal_%02d.txt'%(experiment,FP))
    is_file(fileName)
    columns = 'fp\ttrial\tcond\tlevel\tobj_key\tsub_key\tcorrect\taware\tstair\n'
    with open(fileName, 'w') as f:
        f.write(columns)
    conditions=[{'label':'left', 'startVal': -50, 'nUp':1, 'nDown':1, 'minVal' : -200,
                'maxVal' : -20, 'stepSizes':[8,8,4,4,2,2,1], 'stepType':'lin'},
                {'label':'right', 'startVal': -50, 'nUp':1, 'nDown':1, 'minVal' : -200, 
                'maxVal' : -20, 'stepSizes':[8,8,4,4,2,2,1], 'stepType':'lin'}]
    stairs = data.MultiStairHandler(stairType='simple', method = 'random', conditions=conditions, nTrials=int(n/2))
    aware_ = []
    correct_ = []
    t = 0
    s = load_click()
    ready()
    for level in stairs:
        lvl = level[0]
        stair = level[1]['label']
        if stair == 'left': # left
            itd = -0.25
            ild = -8
            trig = 254
            cond = 0
        elif stair == 'right': # right
            itd = 0.25
            ild = 8
            trig = 253
            cond = 1
        if debug == 0:
            sound = click(s, itd, ild, lvl, nr_channels)
            trial(sound,trig)
            VT.text = instr.objInst
            VT.draw()
            win.flip()
        correct, obj_key = obj_resp(cond, lvl, debug)
        correct_ += [correct]
        if debug == 0:
            VT.text = instr.subInst
            VT.draw()
            win.flip()
        aware, sub_key = sub_resp(cond, lvl, debug)
        aware_ += [aware]
        win.flip()
        stairs.addResponse(aware)
        # Write trial to logfile
        write_log(columns, fileName, FP, t+1, int(cond), lvl, obj_key, int(sub_key), correct, aware, stair)
        t += 1 # count trialnumber

def update_memory(*x):
    txt = ','.join(['{}']*len(x))
    with open(join(wd,'%s_mem.txt'%experiment), 'w') as f:
        f.write(txt.format(*x))

# Prompt
def promt(fp, val, task, lvll, lvlr):
    expInfo = gui.Dlg(title="Auditory detection")
    expInfo.addField('Subject', fp)
    expInfo.addField('Valnr', val)
    expInfo.addText('Experiment Info')
    expInfo.addField('Task', initial=task, choices = ['Validation','Practice','Staircase','Experiment'])
    expInfo.addField('Level L', lvll)
    expInfo.addField('Level R', lvlr)
    ok_data = expInfo.show() # show dialog and wait for OK or Cancel
    if expInfo.OK:  # or if ok_data is not None
        data = change_type(ok_data, ['int', 'int', 'str', 'float', 'float'])
        if any(i>0 for i in data[3:]):
            print('Too high level!')
            core.quit()
        else:
            print('Subject: %s\nTask: %s\nValnr: %s\nLevelL: %s\nLevelR: %s\n' % tuple(data))
            return data
    else:
        print('User cancelled')
        core.quit()

def showpromt(wd):
    # check if there is a memory file
    if isfile(join(wd,'%s_mem.txt'%experiment)):
        print('Loading existing memory...')
        with open(join(wd,'%s_mem.txt'%experiment), 'r') as f:
            data = f.read().split(',')
        FP, valnr, task, lvll, lvlr = promt(*data)
    else:
        print('No memory, making.')
        FP, valnr, task, lvll, lvlr = promt('01','01','Practice', -30, -30)

    # Write memory file
    update_memory(FP, valnr, task, lvll, lvlr)

    # Make subjectfolder if needed
    fp_logdir = join(logdir,'fp%02d'%FP)
    if not isdir(fp_logdir):
        mkdir(fp_logdir)
    return(FP, valnr, task, lvll, lvlr, fp_logdir)

def make_objects():
    # Objects
    win = visual.Window([1920, 1080], allowGUI = False, fullscr = False, #1920, 1080
        monitor = 'testMonitor', units = 'height', gammaErrorPolicy='ignore',
        color='black')
    #Stimuli
    fixation = visual.TextStim(win, color='gray', height = 0.05, text = '+')
    diode = visual.Rect(win=win, name='diodeL', width=0.02, height=0.02,
                         pos=(-0.88, 0.49), fillColor=[1,1,1])
    VT = visual.TextStim(win, color='gray', height=0.02, pos=(0,0), text='')
    return(win, fixation, diode, VT)


# START PROGRAM ===============================================================
while True:
    FP, valnr, task, lvll, lvlr, fp_logdir = showpromt(wd)
    # Procedure
    if task == 'Practice':
        win, fixation, diode, VT = make_objects()
        procedure(n=20, b=1, log=False, proc=task, inst=instr.instTask, lvll=lvll, lvlr=lvlr, catch=0, valnr=valnr, FP=FP, fp_logdir=fp_logdir, task=task)
        ready(instr.endInfo % task)
        update_memory(FP, valnr, 'Staircase', lvll, lvlr)
        win.close()
    elif task == 'Staircase':
        win, fixation, diode, VT = make_objects()
        staircase(90, FP, fp_logdir)
        ready(instr.endInfo % task)
        win.close()
        calibration, vals = load_data(logdir, FP)
        lvll, lvlr = plot_stair_prf(calibration)
        update_memory(FP, valnr, 'Validation', lvll, lvlr)
    elif task == 'Validation':
        win, fixation, diode, VT = make_objects()
        procedure(n=90, b=1, log=True, proc=task, inst=False, lvll=lvll, lvlr=lvlr, catch=0, valnr=valnr, FP=FP, fp_logdir=fp_logdir, task=task)
        ready(instr.endInfo % task)
        win.close()
        calibration, vals = load_data(logdir, FP)
        lvll, lvlr, nv = plot_val_prf(vals)
        update_memory(FP, max(nv)+1, 'Validation', lvll, lvlr)
    elif task == 'Experiment':
        win, fixation, diode, VT = make_objects()
        procedure(n=800, b=8, log=True, proc=task, inst=instr.instExp, lvll=lvll, lvlr=lvlr, catch=1, valnr=valnr, FP=FP, fp_logdir=fp_logdir, task=task)
        ready(instr.endInfo % task)
        win.close()
        fig, ax = plt.subplots(1, 1, figsize=(10, 10))
        plot_awareness_per_block(join(fp_logdir, 'AAN_LRclick_fp%02d_Exp.txt' % FP), ax)
        plt.show()
        update_memory(FP+1, 1, 'Practice', -30, -30)

core.quit()