# -*- coding: utf-8 -*-
"""
Created on Fri Apr 24 22:08:38 2020

@author: Eklund
"""
import matplotlib.pyplot as plt
import matplotlib.lines as mlines
import statsmodels.formula.api as smf
import statsmodels.api as sm
from os.path import join, expanduser
from os import listdir
import pandas as pd
import numpy as np
from scipy.stats import norm

exp = 'AAN_LRclick'
wd = join(expanduser('~'), 'Box sync', exp, 'data', 'log')

def stair(data, c, col, ax):
    guess = np.mean(data['level'][data['trial']>=10])
    ax.plot(data['level'], color = col)
    ax.set_title('%s : %.2f' % (c,guess))
    ax.set_ylabel('dB')
    ax.set_xlabel('Trial number')
    ax.axhline(guess, color='k')
    ax.axvline(10, color='k')
    ax.scatter(data['trial']-1, data['level'], color='k', s=10)

def model(data, c, col, ax):
    M = smf.glm("aware ~ level", data, family=sm.families.Binomial(link=sm.families.links.probit())).fit()
    x = np.linspace(min(data['level']), max(data['level']), 300)
    y = norm.cdf(M.params[0] + M.params[1]*x)
    th = (norm.ppf(0.5)-M.params[0])/M.params[1]
    ax.plot(data['level'], data['aware'], 'ko')
    ax.plot(x,y, col)
    ax.set_title('Threshold %s: %.2f' % (c,th))
    ax.set_xlabel('dB')
    ax.set_ylabel('Percent aware')
    ax.axvline(th, color='k')
    ax.axhline(0.5, color='k')
    return(th)

def val(data, ax):
    nv = np.unique(data['valnr'])[1:]
    y = [np.mean(data['aware'][(data['cond']==c) & (data['valnr']==v)]) for v in nv for c in [0,1]]
    val_lvl = [list(data['level'][(data['valnr']==i) & (data['cond']==c)])[0] for i in nv for c in [0,1]]
    x = [x for v in nv for x in (v-0.25, v+0.25)]
    ax.scatter(x, y, color=['red','blue']*max(nv), marker='o')
    ax.set_ylim(0,1)
    ax.set_xlim(0.5,max(nv)+0.5)
    ax.axhline(0.5, color='k', linestyle='--')
    for i, v in enumerate(val_lvl):
        ax.text(x[i], y[i]+0.05, str(v), horizontalalignment='center')
    left = mlines.Line2D([], [], color='red', marker='o', linestyle='None',
                          markersize=6, label='left')
    right = mlines.Line2D([], [], color='blue', marker='o', linestyle='None',
                              markersize=6, label='right')
    ax.legend(handles=[left, right])
    ax.set_xticks(x)
    ax.set_xticklabels(np.repeat(nv,2))
    ax.set_xlabel('Validation')
    ax.set_ylabel('Percent aware')
    return nv

def plot_stair_prf(data):
    fig, ((ax1, ax2),(ax3, ax4)) = plt.subplots(2, 2, figsize=(12, 8))
    stair(data[data['stair']=='left'], 'left', 'r', ax1)
    stair(data[data['stair']=='right'], 'right', 'b', ax2)
    thl = model(data[data['stair']=='left'], 'left', 'r', ax3)
    thr = model(data[data['stair']=='right'], 'right', 'b', ax4)
    plt.tight_layout()
    plt.show()
    return(np.round(thl,2), np.round(thr,2))

def plot_val_prf(data):
    fig, (ax1, ax2, ax3) = plt.subplots(1, 3, figsize=(12, 4))
    nv = val(data, ax1)
    thl = model(data[data['cond']==0], 'left', 'r', ax2)
    thr = model(data[data['cond']==1], 'right', 'b', ax3)
    plt.tight_layout()
    plt.show()
    return(np.round(thl,2), np.round(thr,2), nv)

def load_data(wd, fp):
    calibration = pd.read_csv(join(wd, 'fp%02d' % fp, '%s_cal_%02d.txt' %(exp, fp)), sep='\t', header=0, index_col=False)
    calibration['valnr'] = pd.Series(np.repeat(0, len(calibration)))
    fps = [f for f in listdir(join(wd, 'fp%02d' % (fp))) if 'Val' in f]
    vals = pd.DataFrame()
    if len(fps) != 0:
        for i, val in enumerate(fps, start=1):
            tmp = pd.read_csv(join(wd, 'fp%02d' % fp, val), sep='\t', skiprows=6, header=0, index_col=False)
            tmp['valnr'] = pd.Series(np.repeat(i, len(tmp)))
            vals = vals.append(tmp[['level','aware','cond','valnr']])
        vals = vals.append(calibration[['level','aware','cond','valnr']])
    return (calibration, vals)


# calibration, vals = load_data(wd, 4) # data = calibration[calibration['stair']=='left']
# thl, thr = plot_stair_prf(calibration)
# thl, thr, nv = plot_val_prf(vals)
