# -*- coding: utf-8 -*-
"""
Created on Sat Jun  6 12:32:56 2020

@author: Eklund
"""
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from matplotlib.animation import FuncAnimation
from matplotlib.lines import Line2D
from os.path import join, expanduser

def animate_exp(i):
    ax.cla()
    fp = 18
    n = 800
    logdir = join(expanduser('~'),'Box Sync', 'AAN_LRclick', 'data','log')
    #logdir = join('Z:\\')
    file = join('fp%i'%fp, 'AAN_LRclick_fp%02d_Exp.txt'%fp)
    try:
        data = pd.read_csv(join(logdir, file), '\t', index_col=False)
    except:
        data = pd.read_csv(join(logdir, file), '\t', index_col=False, skiprows=6)
    filts = {cond+{0:'L', 1:'R'}[lr]:[(data['cond']==lr) & (data['aware']==ac[0]) & (data['correct']==ac[1]), cols]
                 for lr in [0,1]
                     for cond, ac, cols in zip(['ac', 'ai', 'uc', 'ui'], [[1,1], [1,0], [0,1], [0,0]], [['r','r'],['r','none'],['b','b'],['b','none']])}
    ax.set_ylim(0,1)
    ax.set_xlim(0,n)
    ax.set_yticks(ticks=[0.15, 0.30, 0.45])
    ax.set_yticklabels(['Catch', 'Left', 'Right'])
    ax.grid(axis='x')
    fig.suptitle('Fp: %i'%data['fp'][0], size = 20)
    ax.barh(0.6, len(data['trial']), 0.4, align='edge', color='gray', edgecolor='k')
    for cond in list(filts):
        y = {'aL': 0.32, 'uL':0.28, 'aR':0.47, 'uR':0.43}[cond[0]+cond[-1]]
        filt, col = filts[cond]
        ax.scatter(x=data['trial'][filt], y=[y]*sum(filt), edgecolor=col[0], facecolor=col[1], s=10)

    for au, y, filt, col in zip([0,1], [0.13, 0.17], [(data['cond']==2) & (data['aware']==0), (data['cond']==2) & (data['aware']==1)], [['k', 'none'], ['k', 'k']]):
        ax.scatter(x=data['trial'][filt], y=[y]*sum(filt), edgecolor=col[0], facecolor=col[1], s=10)
    text = 'Trials: %i/800\nAware (%%): (L = %.1f, R = %.1f)\nAware correct (%%): (L = %.1f, R = %.1f)\nUnaware correct (%%): (L = %.1f, R = %.1f)\nStrat: L = %i, R = %i' % (len(data['trial']), # Nr trials
                                                                                                           np.mean(data['aware'][data['cond']==0])*100, # % Aware L
                                                                                                           np.mean(data['aware'][data['cond']==1])*100, # % Aware R
                                                                                                           np.mean(data['correct'][(data['cond']==0) & (data['aware']==1)])*100, # % Aware correct L
                                                                                                           np.mean(data['correct'][(data['cond']==1) & (data['aware']==1)])*100, # % Aware correct R
                                                                                                           np.mean(data['correct'][(data['cond']==0) & (data['aware']==0)])*100, # % Unaware correct L
                                                                                                           np.mean(data['correct'][(data['cond']==1) & (data['aware']==0)])*100, # % Unaware correct R
                                                                                                           sum((data['aware']==0) & (data['obj_key']=='left')),
                                                                                                           sum((data['aware']==0) & (data['obj_key']=='right')))
    ax.set_xlabel(text, size=12, ha='left')
    ax.xaxis.set_label_coords(0.03, -0.05)
    legend_elements = [Line2D([0], [0], marker='o', color='w', label='Aware correct', markerfacecolor='r', markeredgecolor='r', markersize=8),
                       Line2D([0], [0], marker='o', color='w', label='Aware incorrect', markerfacecolor='none', markeredgecolor='r', markersize=8),
                       Line2D([0], [0], marker='o', color='w', label='Unaware correct', markerfacecolor='b', markeredgecolor='b', markersize=8),
                       Line2D([0], [0], marker='o', color='w', label='Unaware incorrect', markerfacecolor='none', markeredgecolor='b', markersize=8),
                       Line2D([0], [0], marker='o', color='w', label='Catch aware', markerfacecolor='k', markeredgecolor='k', markersize=8),
                       Line2D([0], [0], marker='o', color='w', label='Catch unaware', markerfacecolor='none', markeredgecolor='k', markersize=8)]
    ax.legend(handles=legend_elements, loc='upper right')

fig, ax = plt.subplots(figsize=(20,5))
fig.suptitle('Fp: X', size = 20)
ani = FuncAnimation(fig, animate_exp, interval=1000)
plt.subplots_adjust(top=0.9,bottom=0.25,left=0.03,right=0.98)
plt.show()
